/***************************************************************************
 *   Copyright (C) 2005 by Mc   *
 *   mc@bglug.it   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "cdialogmusicbrainzfetch.h"


CDialogMusicbrainzFetch::CDialogMusicbrainzFetch(QWidget *parent, const char *name, SONG_STRUCT fetchResults)
    :MusicbrainzFetch(parent, name)
{
  _fetchResults = &fetchResults;

  artist->insertStringList( removeDups( _fetchResults->artist ) );
  album->insertStringList( removeDups( _fetchResults->album  ) );
  title->insertStringList( removeDups( _fetchResults->title  ) );
  trackNum->insertStringList( removeDups( _fetchResults->trackNum ) );
  year->insertStringList( removeDups( _fetchResults->year ) );

  connect( artist , SIGNAL(activated(const QString &)) , this , SLOT(artistActivated(const QString&)) );
  connect( album , SIGNAL(activated(const QString &)) , this , SLOT(albumActivated(const QString&)) );
  connect( title , SIGNAL(activated(const QString &)) , this , SLOT(titleActivated(const QString&)) );
  connect( trackNum , SIGNAL(activated(const QString &)) , this , SLOT(trackNumActivated(const QString&)) );
  connect( year , SIGNAL(activated(const QString &)) , this , SLOT(yearActivated(const QString&)) );

  buttonGroup->adjustSize();
}

#include "cdialogmusicbrainzfetch.moc"


/*!
    \fn CDialogMusicbrainzFetch::accept()
 */
void CDialogMusicbrainzFetch::accept()
{
  QDialog::accept();
}


/*!
    \fn CDialogMusicbrainzFetch::close()
 */
void CDialogMusicbrainzFetch::close()
{
  QDialog::reject();
}


/*!
    \fn CDialogMusicbrainzFetch::removeDups(const &QStringList)
    Pass a QStringList and remove duplicates
 */
QStringList CDialogMusicbrainzFetch::removeDups(const QStringList &srcList)
{
  QStringList dstList;
  for(QStringList::ConstIterator it = srcList.begin() ; it != srcList.end() ; ++it )
    {
      if (dstList.find( (*it) ) == dstList.end() )
        dstList.append( (*it) );
    }
  return dstList;
}


/*!
    \fn CDialogMusicbrainzFetch::syncComboBoxs(QString &choose)
    Syncronize all combo box
 */
void CDialogMusicbrainzFetch::syncComboBoxs( int index )
{
  artist->setCurrentText( *(_fetchResults->artist.at(index)) );
  album->setCurrentText( *(_fetchResults->album.at(index)) );
  title->setCurrentText( *(_fetchResults->title.at(index)) );
  year->setCurrentText( *(_fetchResults->year.at(index))  );
  trackNum->setCurrentText( *(_fetchResults->trackNum.at(index)) );
}


/*!
    \fn CDialogMusicbrainzFetch::albumActivated(int index)
 */
void CDialogMusicbrainzFetch::albumActivated( const QString &choose )
{
  syncComboBoxs(  _fetchResults->album.findIndex( choose ) );
}


/*!
    \fn CDialogMusicbrainzFetch::artistActivated(int index)
 */
void CDialogMusicbrainzFetch::artistActivated( const QString &choose )
{
  syncComboBoxs(  _fetchResults->artist.findIndex( choose ) );
}


/*!
    \fn CDialogMusicbrainzFetch::trackActivated(int index)
 */
void CDialogMusicbrainzFetch::trackNumActivated( const QString &choose )
{
  syncComboBoxs(  _fetchResults->trackNum.findIndex( choose ) );
}


/*!
    \fn CDialogMusicbrainzFetch::titleActivated(int index)
 */
void CDialogMusicbrainzFetch::titleActivated( const QString &choose )
{
  syncComboBoxs(  _fetchResults->title.findIndex( choose ) );
}


/*!
    \fn CDialogMusicbrainzFetch::yearActivated(int index)
 */
void CDialogMusicbrainzFetch::yearActivated( const QString &choose )
{
  syncComboBoxs(  _fetchResults->year.findIndex( choose ) );
}


