/***************************************************************************
 *   Copyright (C) 2005 by Mc                                              *
 *   mc@bglug.it                                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#include <kmp3tag.h>
#include "mimetypeanalyzer.h"



Kmp3Tag::Kmp3Tag()
    : KMainWindow( 0, "Kmp3Tag" )
{

  setAcceptDrops(true);
  QSplitter *split = new QSplitter(QSplitter::Horizontal,this);
  setCentralWidget(split);

  //Polulate Label
  columnLabelCreate();

  //Setting Table Properties
  mainTable = new QTable( split, "mainTable" ); //Crea la tabella
  mainTable->setLeftMargin(0);
  mainTable->setNumCols(labels.count());
  mainTable->setShowGrid(TRUE);
  mainTable->setSorting(FALSE);
  mainTable->setColumnMovingEnabled(TRUE);
  mainTable->setRowMovingEnabled(TRUE);
  mainTable->setSelectionMode( QTable::MultiRow );
  mainTable->updateGeometry();
  mainTable->setReadOnly(FALSE);
  mainTable->setColumnLabels(labels);
  mainTable->hideColumn(columnLabelFinder("Id"));
  mainTable->hideColumn(columnLabelFinder("Path"));

  //Setting column Size
  for( unsigned int i=0; i<labels.count();i++ )
    {
      mainTable->setColumnWidth(i,80);
    }
  // Setting Last Column Streachable
  mainTable->setColumnStretchable(labels.count()-1,TRUE);
  setAutoSaveSettings();
  // Setting Connections
  connect(mainTable,SIGNAL(valueChanged(int, int)), this ,SLOT(tagChange(int, int)));
  connect(mainTable->horizontalHeader(),SIGNAL(clicked(int)),this,SLOT(columnSort(int)));
  connect(mainTable,SIGNAL(clicked(int,int,int,const QPoint &)),this,SLOT(rightClickMenuShow(int,int,int,const QPoint &)));
  // Setting Right Click Header RightClickEvent
  mainTable->horizontalHeader()->installEventFilter( new RightClickEvent() );
  //Create Menu
  mainMenuBuilder();
}

Kmp3Tag::~Kmp3Tag()
{}

void Kmp3Tag::fileOpen()
{
  KURL::List selectedURLs;
  int tableId = 0;
  QString filter("*.mp3 *.ogg *.mpc *.flac| All Multimedia Files (*.mp3, *.ogg, *.mpc, *.flac)\n * | All Files (*.*)");
  statusbar->message("Opening Files");

  selectedURLs = KFileDialog::getOpenURLs(QString::null , filter , this , "File");
  if (!selectedURLs.isEmpty())
    {
      rowsRemoveAll();
      KTagFileRefList.clear();
      for ( KURL::List::Iterator it = selectedURLs.begin(); it != selectedURLs.end(); ++it )
        {
          KTagFileRefList.append( KTagFileRef((*it).path(),FALSE,tableId));
          kTagFileRefToTable(tableId);
          tableId++;
          //MimeTypeAnalyzer *m = new MimeTypeAnalyzer((*it).path());
        }
    }
  statusbar->clear();
}

void Kmp3Tag::fileOpenAdd()
{
  KURL::List selectedURLs;
  int TableId = 0;

  for( int j = (mainTable->numRows()-1); j >= 0  ; j-- )
    {
      if (TableId < mainTable->text(j,0).toInt())
        TableId = mainTable->text(j,0).toInt();
    }
  TableId++;

  QString filter("*.mp3 *.ogg *.mpc *.flac| All Multimedia Files (*.mp3, *.ogg, *.mpc, *.flac)\n * | All Files (*.*)");
  statusbar->message("Opening Files");

  selectedURLs = KFileDialog::getOpenURLs(QString::null , filter , this , "File");
  if (!selectedURLs.isEmpty())
    {
      for ( KURL::List::Iterator it = selectedURLs.begin(); it != selectedURLs.end(); ++it )
        {
          KTagFileRefList.append( KTagFileRef((*it).path(),FALSE,TableId));
          kTagFileRefToTable(TableId);
          TableId++;
        }
    }
  statusbar->clear();
}



void Kmp3Tag::tagChange(int row, int col)
{
  QString fileNewUrl;
  QString fileOldUrl;
  QString cellValue;
  QString currentLabel;
  QRegExp spaces("[\\s]*$");
  QValueList<KTagFileRef>::Iterator obj;
  bool result=FALSE;

  obj = getKTagFileRefIteratorFromRowNum(row);
  cellValue = mainTable->text(row,col);
  cellValue = cellValue.remove( spaces );
  mainTable->setText(row,col,cellValue);
  currentLabel = mainTable->horizontalHeader()->label(col);
  if ( currentLabel == "File" )
    {
      fileNewUrl = (*obj).directory + cellValue;
      fileOldUrl = (*obj).url;
      QDir d;
      result = d.rename(fileOldUrl,fileNewUrl);
      KTagFileRefList.append(KTagFileRef(fileNewUrl, FALSE,(*obj).id));
      KTagFileRefList.remove(obj);
    }

  if (currentLabel == "Track" )
    result = (*obj).setField(KTagFileRef::TRACK,cellValue);
  if (currentLabel == "Year" )
    result = (*obj).setField(KTagFileRef::YEAR,cellValue);
  if (currentLabel == "Genre" )
    result = (*obj).setField(KTagFileRef::GENRE,cellValue);
  if (currentLabel == "Comment" )
    result = (*obj).setField(KTagFileRef::COMMENT,cellValue);
  if (currentLabel == "Title" )
    result = (*obj).setField(KTagFileRef::TITLE,cellValue);
  if (currentLabel == "Album" )
    result = (*obj).setField(KTagFileRef::ALBUM,cellValue);
  if (currentLabel == "Artist" )
    result = (*obj).setField(KTagFileRef::ARTIST,cellValue);

  if (result)
    {
      statusbar->message("Changes status: SUCCESS",1000);
    }
  else
    {
      statusbar->message("Changes status: FAIL",1000);
    }

}

void Kmp3Tag::columnRefresh()
{
  for (unsigned int i=0;i<labels.count();i++)
    {
      mainTable->adjustColumn(i);
    }
}



#include "kmp3tag.moc"


/*!
    \fn Kmp3Tag::mainMenuBuilder()
 */
void Kmp3Tag::mainMenuBuilder()
{
  //Create File menu
  KIconLoader *ic = new KIconLoader ( "kmp3tag");
  fileMenu = new KPopupMenu ( this , "filemenu" );
  fileMenu->insertItem(ic->loadIconSet( "fileopen", KIcon::Small ),"&Open Songs", this , SLOT(fileOpen()), CTRL + Key_O );
  fileMenu->insertItem(ic->loadIconSet( "fileopen", KIcon::Small ),"&Add Songs", this , SLOT(fileOpenAdd()), CTRL + Key_A );
  fileMenu->insertItem(ic->loadIconSet( "fileclose", KIcon::Small ),"&Close All Songs", this , SLOT(rowsRemoveAll()) );
  fileMenu->insertSeparator();
  fileMenu->insertItem(ic->loadIconSet( "exit", KIcon::Small ), "&Exit", this , SLOT(close()));

  //Create Modify menu
  rightClickPopupMenu = new KPopupMenu();
  rightClickPopupMenu->insertItem(ic->loadIconSet( "editcopy", KIcon::Small ),    "Copy", 0 );
  rightClickPopupMenu->setAccel( CTRL + Key_C , 0 );
  rightClickPopupMenu->insertItem(ic->loadIconSet( "editpaste", KIcon::Small ),   "Paste", 1 );
  rightClickPopupMenu->setAccel( CTRL + Key_V , 1 );
  rightClickPopupMenu->insertItem(ic->loadIconSet( "editclear", KIcon::Small ),   "Clear",5);
  rightClickPopupMenu->insertItem(ic->loadIconSet( "kontact_date", KIcon::Small ),"Renumber",2);
  rightClickPopupMenu->insertItem(ic->loadIconSet( "compfile", KIcon::Small ),    "File as Tag",3);
  rightClickPopupMenu->insertItem(ic->loadIconSet( "remove", KIcon::Small ),      "Remove Rows",4);
  rightClickPopupMenu->insertItem(ic->loadIconSet( "next", KIcon::Small ),        "Remove Prefix",6);
  rightClickPopupMenu->insertItem(ic->loadIconSet( "connect_creating", KIcon::Small ),        "MusicBrazinz Fetch",7);
  QObject::connect(rightClickPopupMenu,SIGNAL(activated(int)),this,SLOT(rightClick(int)));


  //Create Help menu
  help = helpMenu();

  //Create View menu
  columnPopupMenu = new KPopupMenu();
  columnPopupMenu->setCheckable(TRUE);
  int i=0;
  for( QStringList::Iterator it = labels.begin(); it != labels.end() ; ++it )
    {
      columnPopupMenu->insertItem(*it,i,i);
      i++;
    }
  QObject::connect(columnPopupMenu,SIGNAL(activated(int)),this,SLOT(columnShowHideExec(int)));
  columnPopupMenu->removeItemAt(0);
  columnPopupMenuRefresh();


  //Create Main Menu
  menu = menuBar();
  menu->insertItem( "&File", fileMenu );
  menu->insertItem( "&Modify", rightClickPopupMenu );
  menu->insertItem( "&View", columnPopupMenu );
  menu->insertItem( "&Help", help );

  //Create StatusBar
  statusbar = statusBar();
  delete (ic);
}


/*!
    \fn Kmp3Tag::columnLabel()
 */
void Kmp3Tag::columnLabelCreate()
{
  labels.append ("Id");
  labels.append ("Path");
  labels.append ("Track");
  labels.append ("Title");
  labels.append ("File");
  labels.append ("Album");
  labels.append ("Artist");
  labels.append ("Year");
  labels.append ("Genre");
  labels.append ("Comment");
}


/*!
    \fn Kmp3Tag::columnLabelFinder(QString)
 */
int Kmp3Tag::columnLabelFinder(QString source)
{

  QString master("");
  for(unsigned int i=0 ; i<labels.count() ; i++ )
    {
      master = mainTable->horizontalHeader()->label(i);
      if (master==source)
        {
          return(i);
        }
    }
  return (-1);
}


/*!
    \fn Kmp3Tag::columnSort()
 */
void Kmp3Tag::columnSort(int a)
{
  static bool m_flagSort = TRUE;
  mainTable->sortColumn(a,m_flagSort,TRUE);
  m_flagSort = !m_flagSort;
}


/*!
    \fn Kmp3Tag::kTagFileRefToTable(int kTagId)
 */

void Kmp3Tag::kTagFileRefToTable(int kTagId)
{
  int     row = 0;
  int     totalRow  = mainTable->numRows();
  bool    idExist = FALSE;
  KTagFileRef *obj;
  QString qTemp;

  for (int i=0;i < totalRow  ;i++)
    {  //find if exist the same row id
      if (kTagId==getKTagFileRefFromRowNum(i)->id)
        {
          row = i;
          idExist = TRUE;
        }
    }
  if (!idExist)
    {//if does not exist create a new row
      if (totalRow == 0)
        {
          mainTable->insertRows(0,1);
          row = 0;
        }
      else
        {
          mainTable->insertRows(totalRow,1);
          row = totalRow;
        }
    }

  mainTable->setText(row,columnLabelFinder("Id"),qTemp.setNum(kTagId));// Cell Object Id
  obj = getKTagFileRefFromRowNum(row);
  if (obj->track == "0")
    {
      mainTable->setText(row,columnLabelFinder("Track"),QString::null);
    }
  else
    {
      mainTable->setText(row,columnLabelFinder("Track"),obj->track);
    }
  if (obj->year == "0")
    {
      mainTable->setText(row,columnLabelFinder("Year"),QString::null);
    }
  else
    {
      mainTable->setText(row,columnLabelFinder("Year"),obj->year);
    }
  mainTable->setText(row,columnLabelFinder("Path"),obj->directory);
  mainTable->setText(row,columnLabelFinder("File"),obj->fileName);
  mainTable->setText(row,columnLabelFinder("Title"),obj->title);
  mainTable->setText(row,columnLabelFinder("Album"),obj->album);
  mainTable->setText(row,columnLabelFinder("Artist"),obj->artist);
  mainTable->setText(row,columnLabelFinder("Genre"),obj->genre);
  mainTable->setText(row,columnLabelFinder("Comment"),obj->comment);
  columnRefresh();
}



/*!
    \fn Kmp3Tag::columnHideMenuShow(QPoint p)
 */
void Kmp3Tag::columnHideMenuShow(QPoint p)
{
  columnPopupMenu->popup(p);
}


/*!
    \fn Kmp3Tag::columnShowHideExec(int)
 */
void Kmp3Tag::columnShowHideExec(int labelId)
{
  int columnId = columnLabelIdToColumnId(labelId);
  if (mainTable->isColumnHidden(columnId))
    {
      mainTable->showColumn(columnId);
      columnPopupMenu->setItemChecked(labelId,TRUE);
    }
  else
    {
      mainTable->hideColumn(columnId);
      columnPopupMenu->setItemChecked(labelId,FALSE);
    }
  columnPopupMenuRefresh();
}


/*!
    \fn Kmp3Tag::columnLabelIdToColumnId(int)
 */
int Kmp3Tag::columnLabelIdToColumnId(int labelId)
{
  QString s = labels[labelId];
  return columnLabelFinder(s.ascii());
}

/*!
    \fn Kmp3Tag::rightClickMenuShow(int,int,int,const QPoint &)
 */
void Kmp3Tag::rightClickMenuShow(int row,int col,int button, const QPoint &p)
{
  if (button==2)
    {
      actualRowPos = row;
      actualColPos = col;
      rightClickPopupMenu->popup(QWidget::mapToGlobal(p));
    }
}


/*!
    \fn Kmp3Tag::rightClick(int)
 */
void Kmp3Tag::rightClick(int choose)
{
  switch (choose)
    {

    case 0: //copyValue
      cellCopy();
      break;

    case 1: //pasteValue
      cellPaste();
      break;

    case 2: //renumber
      tagRenumber();
      break;

    case 3: //fileAsTag
      fileAsTag();
      break;

    case 4: //row remove
      rowsRemove();
      break;

    case 5: //Empty Cell
      cellClear();
      break;

    case 6: //Remove prefix
      rowsRemovePrefix();
      break;

    case 7: //fetch Musicbrainz Info
      musicbrainzFetchSubmit();
      break;

    }
}


/*!
    \fn Kmp3Tag::cellCopy()
 */
void Kmp3Tag::cellCopy()
{
  QString field = dialogChooseField();
  if ( field != QString::null )
    {
      actualColPos = columnLabelFinder(field);
      copyValue = mainTable->text(actualRowPos , actualColPos);
      QString message = copyValue;
      message.prepend("Cell Copied, Value : ");
      statusbar->message(message);
    }
}


/*!
    \fn Kmp3Tag::cellPaste()
 */
void Kmp3Tag::cellPaste()
{
  QMemArray<int> *sel = selectedRows();
  QString field = dialogChooseField();
  if ( field != QString::null )
    {
      actualColPos = columnLabelFinder(field);
      for (QMemArray<int>::Iterator it = sel->
                                         begin() ;
           it != sel->end() ;
           ++it)
        {
          mainTable->setText(*it,actualColPos,copyValue);
          tagChange(*it,actualColPos);
        }
      statusbar->message("Cells Pasted",1000);
    }
  delete (sel);
}


/*!
    \fn Kmp3Tag::tagRenumber()
 */
void Kmp3Tag::tagRenumber()
{
  CDialogRenumber   *dialog;
  QString field, currentColumnLabel;
  QButton *button;
  int progressiveNum;

  dialog = new CDialogRenumber(this);
  currentColumnLabel = mainTable->horizontalHeader()->label(actualColPos);
  for (int i = dialog->buttonGroup->count() -1 ; i >= 0 ; i--)
    {
      button = dialog->buttonGroup->find(i);
      if ( button->text().remove("&") == currentColumnLabel )
        dialog->buttonGroup->setButton(i);
    }
  dialog->exec();
  if (dialog->result()==QDialog::Accepted)
    {
      field = dialog->buttonGroup->selected()->text();
      field = field.remove("&");
      progressiveNum = dialog->spinBoxStart->value();
      statusbar->message("Cells Renumbered",1000);
      for (int i=0;i<mainTable->numRows();i++)
        {
          if (mainTable->isRowSelected(i,FALSE))
            {
              if (field == "Track")
                {
                  QString t;
                  t.setNum(progressiveNum);
                  mainTable->setText(i,columnLabelFinder("Track"),t);
                  this->tagChange(i,columnLabelFinder("Track"));
                };
              if (field == "File")
                {
                  QString actValue = mainTable->text(i,columnLabelFinder("File"));
                  QString t;
                  t.setNum(progressiveNum);
                  if (t.length()<2)
                    {
                      t.prepend("0");
                    }
                  actValue.prepend("-");
                  actValue.prepend(t);
                  mainTable->setText(i,columnLabelFinder("File"),actValue);
                  this->tagChange(i,columnLabelFinder("File"));
                }
              if (field == "Title")
                {
                  QString actValue = mainTable->text(i,columnLabelFinder("Title"));
                  QString t;
                  t.setNum(progressiveNum);
                  if (t.length()<2)
                    {
                      t.prepend("0");
                    }
                  actValue.prepend("-");
                  actValue.prepend(t);
                  mainTable->setText(i,columnLabelFinder("Title"),actValue);
                  this->tagChange(i,columnLabelFinder("Title"));
                }
              progressiveNum++;
            }
        }

    }
  delete(dialog);
}


/*!
    \fn Kmp3Tag::fileAsTag()
 */
void Kmp3Tag::fileAsTag()
{
  QMemArray<int> *sel = selectedRows();
  CDialogFileAsTag  *dialog;
  QString separator,
  firstField,
  secondField,
  thirdField,
  fileType,
  filename;
  int firstFieldID, secondFieldID, thirdFieldID;

  dialog = new CDialogFileAsTag(this, "File As Tag", labels);
  dialog->exec();

  {
    separator   = dialog->separatorLineEdit->text();
    fileType    = dialog->fileTypeComboBox->currentText();

    firstField  = dialog->firstFieldComboBox->currentText();
    if (firstField.isEmpty())
      {
        firstFieldID = 0;
      }
    else
      {
        firstFieldID = columnLabelFinder(firstField);
      }

    secondField = dialog->secondFieldComboBox->currentText();
    if (secondField.isEmpty())
      {
        secondFieldID = 0;
      }
    else
      {
        secondFieldID = columnLabelFinder(secondField);
      }

    thirdField  = dialog->thirdFieldComboBox->currentText();
    if (thirdField.isEmpty())
      {
        thirdFieldID = 0;
      }
    else
      {
        thirdFieldID = columnLabelFinder(thirdField);
      }

    if ((firstFieldID == -1) or (secondFieldID == -1) or (thirdFieldID == -1))
      qDebug ("Field Does not exist");
    for (QMemArray<int>::Iterator it = sel->
                                       begin() ;
         it != sel->end() ;
         ++it)
      {
        filename = "";
        if (firstFieldID)
          {
            if ((firstField == "Track") and (mainTable->text(*it,firstFieldID).length()<2))
              {
                filename +="0";
              }
            filename += mainTable->text(*it,firstFieldID);
          }
        if (secondFieldID)
          {
            filename += separator;
            if ((secondField == "Track") and (mainTable->text(*it,secondFieldID).length()<2))
              {
                filename +="0";
              }
            filename += mainTable->text(*it,secondFieldID);
          }
        if (thirdFieldID)
          {
            filename += separator;
            if ((thirdField == "Track") and (mainTable->text(*it,thirdFieldID).length()<2))
              {
                filename +="0";
              }
            filename += mainTable->text(*it,thirdFieldID);
          }
        filename += fileType;
        filename.replace( QRegExp ( "\\*|/|\\|\'" ), " ");
        mainTable->setText(*it,columnLabelFinder("File"),filename);
        tagChange(*it,columnLabelFinder("File"));
      }
  }
  delete sel;
  delete dialog;
}


/*!
    \fn Kmp3Tag::rowHide()
 */
void Kmp3Tag::rowsRemove()
{
  QMemArray<int> *sel = selectedRows();
  for (QMemArray<int>::Iterator it=sel->
                                   begin();
       it !=sel->end();
       ++it)
    {
      KTagFileRefList.remove(getKTagFileRefIteratorFromRowNum(*it));
    }
  mainTable->removeRows(*sel);
  delete sel;
}


/*!
    \fn Kmp3Tag::rowIdtoObjId()
 */
KTagFileRef* Kmp3Tag::getKTagFileRefFromRowNum(int row)
{
  int id = mainTable->text(row,columnLabelFinder("Id")).toInt();
  for (QValueList<KTagFileRef>::Iterator it = KTagFileRefList.begin(); it != KTagFileRefList.end(); ++it)
    {
      if ((*it).id == id)
        {
          return &(*it);
        }
    }
  return NULL;
}

QValueList<KTagFileRef>::Iterator Kmp3Tag::getKTagFileRefIteratorFromRowNum(int row)
{
  QValueList<KTagFileRef>::Iterator it;
  int id = mainTable->text(row,columnLabelFinder("Id")).toInt();
  for (it = KTagFileRefList.begin(); it != KTagFileRefList.end(); ++it)
    {
      if ((*it).id == id)
        {
          return (it);
        }
    }
  return QValueList<KTagFileRef>::Iterator();
}


/*!
    \fn Kmp3Tag::cellEmpty()
 */
void Kmp3Tag::cellClear()
{
  QMemArray<int> *sel = selectedRows();
  statusbar->message("Cells Cleared",1000);
  for (QMemArray<int>::Iterator it = sel->
                                     begin() ;
       it != sel->end() ;
       ++it)
    {
      mainTable->setText(*it,actualColPos,"");
      tagChange(*it,actualColPos);
    }
  delete sel;
}


/*!
    \fn Kmp3Tag::rowRemovePrefix()
 */
void Kmp3Tag::rowsRemovePrefix()
{
  QMemArray<int> *sel = selectedRows();
  CDialogRemovePrefix *dialog;
  QString field, replaced, currentColumnLabel;
  QButton *button;
  int col;

  dialog = new CDialogRemovePrefix(this);
  currentColumnLabel = mainTable->horizontalHeader()->label(actualColPos);
  for (int i = dialog->buttonGroup->count() -1 ; i >= 0 ; i--)
    {
      button = dialog->buttonGroup->find(i);
      if ( button->text().remove("&") == currentColumnLabel )
        dialog->buttonGroup->setButton(i);
    }
  dialog->exec();
  if(dialog->result()==QDialog::Accepted)
    {
      field = dialog->buttonGroup->selected()->text();
      field = field.remove("&");
      col = columnLabelFinder(field);
      if (col == -1)
        return;
      for (QMemArray<int>::Iterator it = sel->
                                         begin() ;
           it != sel->end() ;
           ++it)
        {
          replaced = mainTable->text(*it,col);
          replaced.remove(dialog->pattern->text());
          mainTable->setText(*it,col,replaced);
          tagChange(*it,col);
        }
    }
  delete sel;
  delete dialog;
}


/*!
    \fn Kmp3Tag::dragEnterEvent(QDragEnterEvent *event)
 */
void Kmp3Tag::dragEnterEvent(QDragEnterEvent *event)
{
  event->accept(QUriDrag::canDecode(event));
}


/*!
    \fn Kmp3Tag::dropEvent(QDropEvent *event)
 */
void Kmp3Tag::dropEvent(QDropEvent *event)
{
  QFileInfo *fileString;
  QStringList dropFileList, fileList;
  if (QUriDrag::decodeLocalFiles(event, dropFileList))
    {
      for (QStringList::Iterator it = dropFileList.begin() ; it != dropFileList.end() ; ++it)
        {
          fileString = new QFileInfo((*it));
          if (fileString->extension().find(QRegExp("mp3|ogg|mpc|flac",FALSE),0) >= 0 )
            {
              fileList.append( *it );
            }
          delete fileString;
        }
      fileOpenAddDragNDrop(fileList);
    }
}


/*!
    \fn Kmp3Tag::selectedRow()
 */
QMemArray<int> *Kmp3Tag::selectedRows()
{
  int j=0;
  QMemArray<int> *selection;
  selection = new QMemArray<int>(mainTable->numRows());
  for (int i=(mainTable->numRows()-1); i >= 0 ; --i)
    {
      if (mainTable->isRowSelected(i,FALSE))
        {
          selection->at(j)=i;
          j++;
        }
    }
  selection->resize(j);
  selection->sort();
  return selection;
}


/*!
    \fn Kmp3Tag::columnPopupMenuRefresh()
 */
void Kmp3Tag::columnPopupMenuRefresh()
{
  int columnId;
  for( unsigned int i=0;i<labels.count() ;++i )
    {
      columnId = columnLabelIdToColumnId(i);
      if (mainTable->isColumnHidden(columnId))
        {
          columnPopupMenu->setItemChecked(i,FALSE);
        }
      else
        {
          columnPopupMenu->setItemChecked(i,TRUE);
        }
    }
}


/*!
    \fn Kmp3Tag::fileOpenAddDragNDrop(QStringList files)
 */
void Kmp3Tag::fileOpenAddDragNDrop(QStringList files)
{
  int TableId = 0;
  for( int j = (mainTable->numRows()-1); j >= 0  ; j-- )
    {
      if (TableId < mainTable->text(j,0).toInt())
        TableId = mainTable->text(j,0).toInt();
    }
  TableId++;

  for ( QStringList::Iterator it = files.begin(); it != files.end(); ++it )
    {
      KTagFileRefList.append( KTagFileRef(*it ,FALSE,TableId));
      kTagFileRefToTable(TableId);
      TableId++;
    }


  statusbar->clear();
}


/*!
    \fn Kmp3Tag::rowsRemoveAll()
 */
void Kmp3Tag::rowsRemoveAll()
{
  QMemArray<int> allRows(mainTable->numRows());
  for(int i = mainTable->numRows() - 1 ; i >= 0 ; --i)
    allRows.at(i) = i;
  KTagFileRefList.clear();
  mainTable->removeRows(allRows);
}


/*!
    \fn Kmp3Tag::dialogChooseField()
 */
QString Kmp3Tag::dialogChooseField()
{
  QString currentColumnLabel,
  selectedField;
  QButton *button;
  CDialogChooseField *dialog = new CDialogChooseField( this , "Dialog Choose Field" , labels);
  currentColumnLabel = mainTable->horizontalHeader()->label(actualColPos);
  for (int i = dialog->buttonGroup->count() -1 ; i >= 0 ; i--)
    {
      button = dialog->buttonGroup->find(i);
      if ( button->text().remove("&") == currentColumnLabel )
        dialog->buttonGroup->setButton(i);
    }
  dialog->exec();
  if (dialog->result()==QDialog::Accepted)
    {
      selectedField = dialog->buttonGroup->selected()->text().remove("&");
    }
  else
    {
      selectedField = QString::null;
    }

  delete (dialog);
  return (selectedField);
}


/*!
    \fn Kmp3Tag::musicbrainzFetch()
 */
void Kmp3Tag::musicbrainzFetchSubmit()
{
  int objId = mainTable->text(actualRowPos , 0).toInt();
  QString fileUrl = KTagFileRefList[objId].url;
  trm = new KTRMLookup( fileUrl , true );
  // bisogna far partire un thread di timeout
  connect( trm , SIGNAL(sigResult( KTRMResultList , QString )) , SLOT(musicbrainzFetchResult( KTRMResultList , QString )) );
  statusbar->message( "Music Brainz Fetching" );
  QApplication::setOverrideCursor( QCursor(Qt::WaitCursor) );
}


/*!
    \fn Kmp3Tag::musicbrainzFetchResult( KTRMResultList resultList , QString error)
 */
void Kmp3Tag::musicbrainzFetchResult( KTRMResultList resultList , QString error)
{
  QApplication::restoreOverrideCursor();
  QString tmp;
  SONG_STRUCT myResult;
  CDialogMusicbrainzFetch *dialog;
  statusbar->clear();
  if ( !error.isEmpty() )
    {
      statusbar->message( " Error MusicBrainz Fetching " );
      //qDebug( "l errore  stato " + error );
    }
  else
    {
      //       qDebug ( "non ci sono stati errori" );
      //       qDebug ( " gli elementi trovati sono : %i" , resultList.size() );
      if ( resultList.size() > 0)
        {
          for( KTRMResultList::Iterator it = resultList.begin() ; it != resultList.end()  ;  ++it )
            {
              myResult.artist.append((*it).artist());
              myResult.album.append((*it).album());
              myResult.title.append((*it).title());
              if ( (*it).track() > 0 )
                myResult.trackNum.append( tmp.setNum( (*it).track() ) );
              else
                myResult.trackNum.append ( "" );

              if ( (*it).year() > 1900 )
                  myResult.year.append( tmp.setNum( (*it).year() ) );
              else
                myResult.year.append( "" );
            }

          dialog = new CDialogMusicbrainzFetch( this , "MusicBrazinz Fetch" , myResult);
          dialog->exec();
          if (dialog->result()==QDialog::Accepted)
            {
              if ( dialog->checkBoxArtist->isChecked() )
                {
                  mainTable->setText( actualRowPos,columnLabelFinder("Artist"), dialog->artist->currentText() );
                  tagChange( actualRowPos,columnLabelFinder("Artist") );
                }
              if ( dialog->checkBoxAlbum->isChecked() )
                {
                  mainTable->setText( actualRowPos,columnLabelFinder("Album"), dialog->album->currentText() );
                  tagChange( actualRowPos,columnLabelFinder("Album") );
                }
              if ( dialog->checkBoxTitle->isChecked() )
                {
                  mainTable->setText( actualRowPos,columnLabelFinder("Title"), dialog->title->currentText() );
                  tagChange( actualRowPos,columnLabelFinder("Title") );
                }
              if ( dialog->checkBoxTrackNum->isChecked() )
                {
                  mainTable->setText( actualRowPos,columnLabelFinder("Track"), dialog->trackNum->currentText() );
                  tagChange( actualRowPos,columnLabelFinder("Track") );
                }
              if ( dialog->checkBoxYear->isChecked() )
                {
                  mainTable->setText( actualRowPos,columnLabelFinder("Year"), dialog->year->currentText() );
                  tagChange( actualRowPos,columnLabelFinder("Year") );
                }
            }
        }
      else
        {
          statusbar->message( " Element not found  " );
        }
    }
}
