/***************************************************************************
 *   Copyright (C) 2005 by Mc   *
 *   mc@bglug.it   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "ktagfileref.h"
#include <mpegfile.h>
#include <flacfile.h>
#include <oggfile.h>
#include <vorbisfile.h>
#include <mpcfile.h>
#include <qregexp.h>

/**
 *
 * @param f  FileName
 * @param audioAnalysis Boolean Value for Auto Analysis of the track
 * @param TableId ID reference of the tables
 * @return None
 */
KTagFileRef::KTagFileRef(const char *f, bool audioAnalysis,int TableId)
{
  this->audioAnalysis = audioAnalysis;
  if (!fileOpen( f ))
    {
      //Check if system is unix and find file command
      // parse the result of file ask a confirmation to change extension

    }

  spaceRegexp.setPattern("[\\s]*$");
  id = TableId;
  infUpdate(ALL);//update QString field
  spacesRemover( album );
  spacesRemover( artist );
  spacesRemover( comment );
  spacesRemover( genre );
  spacesRemover( title );
  tag->setAlbum(album.ascii());
  tag->setArtist(artist.ascii());
  tag->setComment(comment.ascii());
  tag->setGenre(genre.ascii());
  tag->setTitle(title.ascii());
  //infUpdate(ALL);//update QString field Probably to remove

}


KTagFileRef::~KTagFileRef()
{
//   delete ( tagFile ); // I dont know it crash
}


/*!
    \fn KTagFileRef::infUpdate(int field)
 */
void KTagFileRef::infUpdate(int field)
{
  switch (field)
    {
    case ALBUM :
      album = tag->album().toCString();
      break;
    case ARTIST :
      artist = tag->artist().toCString();
      break;
    case COMMENT :
      comment = tag->comment().toCString();
      break;
    case GENRE :
      genre = tag->genre().toCString();
      break;
    case TITLE :
      title = tag->title().toCString();
      break;
    case TRACK :
      track = track.setNum(tag->track());
      break;
    case YEAR :
      year = year.setNum(tag->year());
      break;
    case URL :
      url       = tagFile->name();
      qUrl      = url;
      fileName  = qUrl.fileName();
      directory = qUrl.dirPath() += "/";
      break;
    case ALL :
      album = tag->album().toCString();
      artist = tag->artist().toCString();
      comment = tag->comment().toCString();
      genre = tag->genre().toCString();
      title = tag->title().toCString();
      track = track.setNum(tag->track());
      year = year.setNum(tag->year());
      url       = tagFile->name();
      qUrl      = url;
      fileName  = qUrl.fileName();
      directory = qUrl.dirPath() += "/";
      break;
    }
}

/*!
    \fn KTagFileRef::spacesRemover(QString *str)
 */
void KTagFileRef::spacesRemover(QString &str)
{
  str.remove( spaceRegexp );
}

/*!
    \fn KTagFileRef::setField(int field, QString str)
 */
bool KTagFileRef::setField(FIELDENUM field, QString value)
{
  int result = FALSE;
  //if (str.find( spaceRegexp ) != -1){str = str.remove( spaceRegexp );} probably to remove
  value.remove( spaceRegexp );
  switch (field)
    {
    case ALBUM :
      tag->setAlbum(value.ascii());
      break;
    case ARTIST :
      tag->setArtist(value.ascii());
      break;
    case COMMENT :
      tag->setComment(value.ascii());
      break;
    case GENRE :
      tag->setGenre(value.ascii());
      break;
    case TITLE :
      tag->setTitle(value.ascii());
      break;
    case TRACK :
      tag->setTrack(value.toInt());
      break;
    case YEAR :
      tag->setYear(value.toInt());
      break;
    }
  result = tagFile->save();
  infUpdate(field);
  return ( result );
}


/*!
    \fn KTagFileRef::fileOpen
    Check the file externsion and initialize Tag Object
 */
bool KTagFileRef::fileOpen( const char* filePath )
{
  QRegExp mpeg("(\\.mp3)$");
  QRegExp mpc("(\\.mpc)$");
  QRegExp flac("(\\.flac)$");
  QRegExp ogg("(\\.ogg)$");
  QString qfilePath(filePath);

  if (qfilePath.find(mpeg))
    {
      tagFile = new TagLib::MPEG::File( filePath , audioAnalysis );
      tag = tagFile->tag();
      return true;
    }

  if (qfilePath.find(mpc))
    {
      tagFile = new TagLib::MPC::File( filePath , audioAnalysis );
      tag = tagFile->tag();
      return true;
    }

  if (qfilePath.find(flac))
    {
      tagFile = new TagLib::FLAC::File( filePath , audioAnalysis );
      tag = tagFile->tag();
      return true;
    }

  if (qfilePath.find(ogg))
    {
      tagFile = new TagLib::Ogg::Vorbis::File( filePath , audioAnalysis );
      tag = tagFile->tag();
      return true;
    }

  return false;
}


/*!
    \fn KTagFileRef::getField(FIELDNUM)
 */
QString KTagFileRef::getField(FIELDENUM field)
{
    /// @todo implement me
}


